/*******************************************************************************
* Title: HT1632C_Setup.c                                                       
* Version: 1.0                                                                 
* Author: Jeff Nybo                                                            
* Date: April 23, 2014                                                      
*                                                                              
* Description:                                                                 
* This file contains the functions which will initialize and control a HT1632C 
* 32x8/24x16 LED Driver. It was initially set up to control one master and one 
* slave chip, however with a bit of modification to the files, it can support  
* more.                                                                        
*******************************************************************************/

#ifndef HT1632C_SETUP_C
#define HT1632C_SETUP_C

#include "Main_Includes.h"
#include "HT1632C_Setup.h"  

/*******************************************************************************
* Function: HT1632_Chip_Select(INT8 selection)                                  
*                                                                              
* Variables:                                                                    
* selection -> Holds the value of the selected chip                             
*                                                                               
* Description:                                                                  
* This function will enable the selected chip for reading/writing or deselect   
* all of the chips for reading/writing. Writing a '0' to this function will     
* deselect all chips, whereas a specific value will select that corresponding   
* chip. If the selected chip is not in the statement, it will deselect all      
* chips by default, otherwise a specific chip can be deselected by writing      
* in the negative value of the selected chip. This function supports up to a    
* maximum of two chips right now, although more could be added.                 
*******************************************************************************/
void HT1632_Chip_Select(INT8 selection)
{
  //Check which chip is to be selected/deselected
  switch (selection)
  {
    //If it is a negative value, the corresponding chip will be deselected
    case -1:  HT1632_CS1 = 1; break;
    case -2:  HT1632_CS2 = 1; break;
    
    //If a 0 is written, deselect all chips
    case 0:  HT1632_CS1 = 1; 
             HT1632_CS2 = 1; 
             break;
    
    //If it is a positive value, select the corresponding chip
    case 1:  HT1632_CS1 = 0; break;
    case 2:  HT1632_CS2 = 0; break;
    
    //If it is an unknown value, deselect all chips
    default: HT1632_CS1 = 1; 
             HT1632_CS2 = 1; 
             break;
  }  
} 

/*******************************************************************************
* Function: HT1632_Init(void)                                                   
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function will initialize the HT1632C(s), starting with the master chip. 
* It will then initialize the remaining amount of HT1632C chips, which are       
* defined in the HT1632C_Setup.h file.                                         
*******************************************************************************/
void HT1632_Init(void)
{
  int i; //Used as a counting variable
  
  //Disable all HT1632 chips, and put READ/WRITE pins as default value
  HT1632_Chip_Select(0);
  HT1632_WR  = 1;
  HT1632_RD  = 1;
  
  //Set chip 1 in Master Mode and disable it; Set COM Option
  HT1632_Send_CMD(1,HT1632_SYSDIS);
  HT1632_Send_CMD(1,HT1632_COMS00);
  HT1632_Send_CMD(1,HT1632_MSTMD);
  
  //Set the slave chip(s) in Slave Mode and disable them
  for (i = 2;i <= HT1632_AMOUNT;i++)
  {
    HT1632_Send_CMD(i,HT1632_SYSDIS);
    HT1632_Send_CMD(i,HT1632_COMS00);
    HT1632_Send_CMD(i,HT1632_SLVMD); 
  }  
  
  //Enable the Master chip and turn on the LEDs
  HT1632_Send_CMD(1,HT1632_SYSON);
  HT1632_Send_CMD(1,HT1632_LEDON);
   
  //Enable the Slave chip(s) and turn on the LEDs
  for (i = 2;i <= HT1632_AMOUNT;i++)
  {
    HT1632_Send_CMD(i,HT1632_SYSON);
    HT1632_Send_CMD(i,HT1632_LEDON); 
  }  
}  

/*******************************************************************************
* Function: HT1632_Send_CMD(INT8 chip, UINT8 cmd)                                
*                                                                               
* Variables:                                                                    
* chip -> The chip to be selected                                  
* cmd -> The command that is to be sent to the HT1632C                                                                       
*                                                                               
* Description:                                                                  
* This function will write a specific command to a specific HT1632C driver. The
* command list is located in the HT1632C datasheet.                                                                              
*******************************************************************************/
void HT1632_Send_CMD(INT8 chip, UINT8 cmd)
{ 
  //Select the chip to write to
  HT1632_Chip_Select(chip);
  
  //Inform the chip that a COMMAND is to be executed
  HT1632_Write_Bits(HT1632_ID_CMD,0x04);
  
  //Write the COMMAND to the chip with the last bit being a dummy bit (0)
  HT1632_Write_Bits(cmd,0x80);
  HT1632_Write_Bits(0,1);
  
  //Deselect the chip
  HT1632_Chip_Select(-chip);
}

/*******************************************************************************
* Function: HT1632_Send_Data(INT8 chip, UINT8 addr, UINT8 data)
*                                                                               
* Variables:                                                                    
* chip -> The chip to be selected
* addr -> The register address that the data is to be written to
* data -> The data that is to be written to the HT1632C                                                                       
*                                                                               
* Description:                                                                  
* This function will send data to a specified address in the HT1632C driver.                                                                             
*******************************************************************************/
void HT1632_Send_Data(INT8 chip, UINT8 addr, UINT8 data)
{
  //Select the chip to write to
  HT1632_Chip_Select(chip);
  
  //Inform the chip that a WRITE is to be executed
  HT1632_Write_Bits(HT1632_ID_WR,0x04);
    
  //Write the data to the specified address in the HT1632C
  HT1632_Write_Bits(addr,0x40);
  HT1632_Write_Bits(data,0x08);
  
  //Deselect the chip
  HT1632_Chip_Select(-chip);
}  

/*******************************************************************************
* Function: HT1632_Set_PWM(UINT8 value)                                                                     
*                                                                               
* Variables:                                                                    
* value -> a value from 0 - 15 which will set the brightness of the grid                                                                           
*                                                                               
* Description:                                                                  
* This function allows the user to set 16 different brightnesses for the grid.                                                                              
*******************************************************************************/  
void HT1632_Set_PWM(UINT8 value)
{
  //Send the PWM Duty ID and the PWM duty value
  HT1632_Send_CMD(1, HT1632_PWM | value);
  HT1632_Send_CMD(2, HT1632_PWM | value);
}

/*******************************************************************************
* Function: HT1632_Write_Bits(UINT8 amt, UINT8 bits)                             
*                                                                               
* Variables:                                                                    
* amt -> The amount of bits that are to be written                                                                        
* bits -> The byte which contains the bits to be written                                                                      
*                                                                               
* Description:                                                                  
* This function will write the specified amount of bits to the HT1632C.                                                                              
*******************************************************************************/
void HT1632_Write_Bits(UINT8 amt, UINT8 bits)
{
  //As long as there are still bits to write, keep writing
  while (bits)
  {
    //Write the bit into the HT1632C, determine whether it is a 1 or a 0
    HT1632_WR = 0;
    HT1632_DATA = (amt & bits) ? 1 : 0;
    HT1632_WR = 1;
    
    //After every write, shift out written bit and continue to the next bit
    bits >>= 1;
  }  
} 
	
#endif
