/*******************************************************************************
* Title: PORT_Setup.c                                                          
* Version: 1.0                                                                 
* Author: Jeff Nybo                                                            
* Date: April 23, 2014                                                      
*                                                                              
* Description:                                                                 
* This file contains the functions that are used for setting up and            
* initializing the ports and register.                                         
*******************************************************************************/

#ifndef PORT_SETUP_C
#define PORT_SETUP_C

#include "Main_Includes.h"
#include "PORT_Setup.h"
#include "ADC_Setup.h"
#include "SPI_Setup.h"
#include "PWM_Setup.h"
#include "Timers_Setup.h"

/*******************************************************************************
* Function: PORT_Init()                                                         
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Pinouts:                                                                     
*                                                                              
* RB00 -> IR_AN0....Connected to infrared sensors on the MASTER side                                                                      
* RB01 -> IR_AN1....Connected to infrared sensors on the SECONDARY side                                                                      
* RB02 -> VU_AN2....Connected to mic audio output for detecting sound                                                                     
* RB03 -> POT_AN3...Connected to ANALOG POT; Used for various adjustments in code                                                                      
* RB04 -> AN4.......Extra analog input                                                                      
* RB05 -> AN5.......Extra analog input                                                                      
* RB06 -> PGC.......Used for ICSP. Programming clock line.                                                                      
* RB07 -> PGD.......Used for ICSP. Programming data line.                                                                 
* RB08 -> NRF_CE....Controls the chip enable pin for RX or TX mode on the nRF24L01                                                                     
* RB09 -> NRF_CSN...Controls the chip select for the nRF24L01 (Active Low)                                                                      
* RB10 -> ---------------- Used to be NRF_IRQ; Now moved to RB11 -----------------                                                                      
* RB11 -> NRF_IRQ...Controls the IRQ interrupt pin on the nRF24L01                                                                     
* RB12 -> N/C                                                                      
* RB13 -> A_IN......Bit #0 (LSb) of the 3 input select bits for the 74HC4051 modules                                                                     
* RB14 -> B_IN......Bit #1 of the 3 input select bits for the 74HC4051 modules                                                                      
* RB15 -> C_IN......Bit #2 (MSb) of the 3 input select bits for the 74HC4051 modules                                                                  
*                                                                               
* RC01 -> N/A                                                                      
* RC02 -> TLC_EN....Controls the enable pin on the SPI buffer for the TLC5940's (Active Low)                                                                       
* RC12 -> OSC1......Oscillator Input                                                                      
* RC13 -> IX2_CS2...Chip select for 2nd 74HC4051 chip on SECONDARY side                                                                      
* RC14 -> N/A                                                                      
* RC15 -> OSC2......Oscillator Input                                                                   
*                                                                              
* RD00 -> IX2_CS1...Chip select for 1st 74HC4051 chip on SECONDARY side                                                                       
* RD01 -> GSCLK.....Grayscale clock used for all 74HC5940 chips (PWM Output)                                                                      
* RD02 -> IX1_CS1...Chip select for 1st 74HC4051 chip on MASTER side                                                                       
* RD03 -> IX1_CS2...Chip select for 2nd 74HC4051 chip on MASTER side                                                                        
* RD04 -> N/C                                                                      
* RD05 -> N/C                                                                      
* RD06 -> N/C                                                                      
* RD07 -> N/C                                                                 
* RD08 -> HT_WR.....Controls the WRITE input on the HT1632 chips (LED Grid)                                                                      
* RD09 -> HT_RD.....Controls the READ input on the HT1632 chips (LED grid)                                                                      
* RD10 -> HT_CS1....Chip select for HT1632 #1 (LEDs: Rows #0 - #32; Columns #0 - #7)                                                                      
* RD11 -> HT_CS2....Chip select for HT1632 #2 (LEDs: Rows #0 - #32; Columns #8 - #11)                                                                  
*                                                                              
* RF00 -> IX2_TX....Controls IR transmitters on the SECONDARY side for the IR sensors (Active High) 
* RF01 -> IX1_TX....Controls IR transmitters on the MASTER side for the IR sensors (Active High)                                                                      
* RF02 -> MISO_EX...Extra SPI module on connector SL1; This SPI module is not currently in use.                                                                     
* RF03 -> MOSI_EX...Extra SPI module on connector SL1; This SPI module is not currently in use.                                                                      
* RF04 -> N/C                                                                      
* RF05 -> N/C                                                                      
* RF06 -> SCK_EX....Extra SPI module on connector SL1; This SPI module is not currently in use.                                                                  
*                                                                              
* RG00 -> DCPRG....Controls the source of the current limit register                                                                      
* RG01 -> N/C                                                                      
* RG02 -> HT_DATA..Controls the DATA input on the HT1632 chips (LED Grid)                                                                         
* RG03 -> N/C                                                                      
* RG06 -> SCK......Clock output for SPI module                                                                      
* RG07 -> MISO.....Master In; Slave Out for SPI module #2 (We use this module for TLC5940's and RF)                                                               
* RG08 -> MOSI.....Master Out; Slave In for SPI module #2 (We use this module for TLC5940's and RF)                                                                     
* RG09 -> N/C                                                                      
* RG12 -> BLANK....Blanking input for TLC5940's. Used to mark the end of a GSCLK cycle.                                                                      
* RG13 -> XLAT.....Latches the data after it has been shifted into the TLC5940's                                                                      
* RG14 -> VPRG.....Used to select CL registers or DC registers for writing in the TLC5940's                                                                      
* RG15 -> N/C                                                                      
*                                                                              
* **All N/C pins are to be set as low outputs                                        
*                                                                              
* Description:                                                                 
* This routine will initialize the ports as needed according to the schematic. 
* This should only be called once in a program.                                
*******************************************************************************/
void PORT_Init(void)
{
 //Configure clock and timing for the MCU
 OSCCON	=	0x2200;	 //HS_PLL Clock
 CLKDIV	=	0x0004;	//do not divide
 PLLFBD = 46;
 
 //Set PORTs as either HIGH or LOW
 LATB = 0xF300; 
 LATC = 0x0000; 
 LATD = 0x0000; 
 LATF = 0x0000; 
 LATG = 0x5001; 

 //Configure PORTs as INPUTS our OUTPUTS
 TRISB = 0X0C3F;
 TRISC = 0X0000;
 TRISD = 0X0000;
 TRISF = 0X0000;
 TRISG = 0X0080; 
}

/*******************************************************************************
* Function: Modules_Init(void)                                                                    
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function is used to initialize the modules/peripherals in the CPU.                                                                             
*******************************************************************************/
void Modules_Init(void)
{
  ADC_Init();
     
  PWM2_Init(6,10);
  TMR2_Init();  //Enable Timer2 for PWM
  TMR1_Init();  //Enable Timer1 
  TMR3_Init();  //Enable Timer3
//  TMR5_Init();  //Enable Timer5
}

#endif
