/*******************************************************************************
* Title: RF_Controls.c                                                                      
* Version: 1.0                                                                 
* Author: Jeff Nybo                                                            
* Date: April 23, 2014                                                                       
*                                                                              
* Description:                                                                 
* This file contains the functions and commands that are necessary in order to                                                                             
* be able to communicate between the beer pong table and a PC.                                                                             
*******************************************************************************/

#ifndef RF_CONTROLS_C
#define RF_CONTROLS_C

/*******************************************************************************
* Function: Tx_Check_CMD(void)                                                                    
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function will read the received data from the nRF24L01+ and parse it to
* determine what command is to be carried out.                                                                             
*******************************************************************************/
void Tx_Check_CMD(void)
{ 
  UINT16 CMD;
	
  //Pack the first 2 USB_Rx bytes which are the COMMAND bytes
  CMD = ((UINT16)USB_Rx[1] << 8) | USB_Rx[0];
  
  //Determine which command is to be carried out and start transmitting data
  switch (CMD)
  {
	  case CMD_REFRESH_SCREEN:    Tx_Refresh_Grid(USB_Rx);     break;
	  case CMD_UPDATE_PODS:       Tx_Update_Pods(USB_Rx);      break;
	  case CMD_UPDATE_RINGS:      Tx_Update_Rings(USB_Rx);     break;
	  case CMD_SCROLLING_TEXT:    Tx_Scrolling_Text(USB_Rx);   break;
	  case CMD_ACTIVATE_RF_MODE:  Tx_Activate_RF_Mode(USB_Rx); break;
	  case CMD_DISABLE_RF_MODE:   Tx_Disable_RF_Mode(USB_Rx);  break;
	}
}

/*******************************************************************************
* Function: Tx_Initialize_RF_Module(void)                             
*                                                                             
* Variables:    
* N/A
*                                                                             
* Description:                                                                
* This function is used to initialize the nRF24L01+ and set its TX and RX
* addresses. These values must match the settings of the RF Master Board to
* enable communication.                                                
*******************************************************************************/          
void Tx_Initialize_RF_Module(void)
{
  UINT8 pipe0[5] = {0xA0,0x00,0x00,0x00,0x00}; 
  UINT8 pipe1[5] = {0x01,0x12,0x12,0x12,0x3F}; 
  UINT8 rx_payloadi[6] = {32,32,32,32,32,32};

	//Initialize the nRF24L01 in RX mode, turn on enhanced shockburst
	//and enable Enhanced Shockburst, set transmission speed to 2Mbps   
  nRF24L01_Init(0x38,1,0x3F,0x3F,0x03,0x2F,0x02,0x07);
  
  //Set up the RX addresses
  nRF24L01_SetRxAddress(pipe0,pipe1,5,rx_payloadi);
  
  //Set up the TX addresses
  nRF24L01_SetTxAddress(pipe0,5); 
}

/*******************************************************************************
* Function: Tx_Refresh_Grid(UINT8 *data)                                      
*                                                                             
* Variables:                                                                  
* *data -> The data that is to be written to the display           
*                                                                             
* Description:                                                                
* This function will parse the data that is to be written to the BPT and send 
* it over two packets. The receiving BPT will then parse the data and display 
* the transmitted data. If the transmission is successful, the function will 
* return a 0. Otherwise it will return an error code.                                                     
*******************************************************************************/ 
UINT8 Tx_Refresh_Grid(UINT8 *data)
{ 
  UINT8 check;

  //Turn on the transmission indicator
  PORTDbits.RD0 = 1;
  
  //Send the first half of the first USB packet
  check = nRF24L01_Tx_SB(&data[0],32);  
 
  //If an ACK was not received, return an error
  if (check)
    return RF_ERROR_NO_ACK; 
  
  //Send the next half of the USB packet 
  check = nRF24L01_Tx_SB(&data[32],32);  
   
  //If an ACK was not received, return an error
  if (check)
    return RF_ERROR_NO_ACK; 
  
  //Re-arm the OUT endpoint for the next packet
  USBOutHandle = HIDRxPacket(HID_EP,(BYTE*)&USB_Rx,64);
  
  while(HIDRxHandleBusy(USBOutHandle));
  
  //Receive the last USB packet, we only use the frist two bytes 
  check = nRF24L01_Tx_SB(USB_Rx,32); 
   
  //If an ACK was not received, return an error
  if (check)
    return RF_ERROR_NO_ACK;   
            
  //Turn off the transmission indicator
  PORTDbits.RD0 = 0;
  
  //Operation completed successfuly, return a 0  
  return 0x00;
}

/*******************************************************************************
* Function: Tx_Update_Pods(UINT8 *data)                                     
*                                                                             
* Variables:                      
* *data -> The data that is to be written to the RGB pods  
*                                                                             
* Description:                                                         
* This function will parse the data that is to be written to the BPT and send 
* it over two packets. The receiving BPT will then parse the data and display 
* the transmitted data across the RGB pods. If the transmission is successful,
* the function will return a 0. Otherwise it will return an error code.                                      
*******************************************************************************/ 
UINT8 Tx_Update_Pods(UINT8 *data)
{ 
  UINT8 check;

  //Turn on the transmission indicator
  PORTDbits.RD0 = 1;
   
  //Send the first half of the first USB packet
  check = nRF24L01_Tx_SB(&data[0],32);  
 
  //If an ACK was not received, return an error
  if (check)
    return RF_ERROR_NO_ACK; 
    
  //Send the second half of the first USB packet
  check = nRF24L01_Tx_SB(&data[32],32);  
   
  //If an ACK was not received, return an error
  if (check)
    return RF_ERROR_NO_ACK; 
       
  //Turn off the transmission indicator  
  PORTDbits.RD0 = 0;
  
  //Operation completed successfuly, return a 0  
  return 0x00;
}

/*******************************************************************************
* Function:                                      
*                                                                             
* Variables:                        
* *data -> The data that is to be written to the LED rings 
*                                                                             
* Description:                                                          
* This function will parse the data that is to be written to the BPT and send 
* it over two packets. The receiving BPT will then parse the data and display 
* the transmitted data on the LED rings. If the transmission is successful,
* the function will return a 0. Otherwise it will return an error code.                                    
*******************************************************************************/ 
UINT8 Tx_Update_Rings(UINT8 *data)
{ 
  UINT8 check;

  //Turn on the transmission indicator
  PORTDbits.RD0 = 1;  
   
  //Send the first half of the first USB packet (We only need to send 26 bytes
  //which will fit into one packet)
  check = nRF24L01_Tx_SB(&data[0],32);  
 
  //If an ACK was not received, return an error
  if (check)
    return RF_ERROR_NO_ACK; 
      
  //Turn off the transmission indicator
  PORTDbits.RD0 = 0;
  
  //Operation completed successfuly, return a 0  
  return 0x00;
} 

/*******************************************************************************
* Function:                                      
*                                                                             
* Variables:                       
* *data -> The data that contains the CMD_ACTIVATE_RF_MODE command
*                                                                             
* Description:                                                          
* This function will transmit the command to activate RF mode on the beer pong
* table. If the transmission is successful, the function will return a 0. 
* Otherwise it will return an error code.                                         
*******************************************************************************/ 
UINT8 Tx_Activate_RF_Mode(UINT8 *data)
{ 
  UINT8 check;

  //Turn on the transmission indicator
  PORTDbits.RD0 = 1;  
   
  //Send the first half of the first USB packet (We only need to send 2 bytes
  //which will fit into one packet)
  check = nRF24L01_Tx_SB(&data[0],32);  
 
  //If an ACK was not received, return an error
  if (check)
    return RF_ERROR_NO_ACK; 
  
  //Turn off the transmission indicator    
  PORTDbits.RD0 = 0;
  
  //Operation completed successfuly, return a 0  
  return 0x00;
}

/*******************************************************************************
* Function:                                      
*                                                                             
* Variables:   
* *data -> The data that contains the CMD_DISABLE_RF_MODE command
*                                                                             
* Description:                                                                
* This function will transmit the command to disable RF mode on the beer pong
* table. If the transmission is successful, the function will return a 0. 
* Otherwise it will return an error code.                                                    
*******************************************************************************/ 
UINT8 Tx_Disable_RF_Mode(UINT8 *data)
{ 
  UINT8 check;

  //Turn on the transmission indicator
  PORTDbits.RD0 = 1;  
   
  //Send the first half of the first USB packet (We only need to send 2 bytes
  //which will fit into one packet)
  check = nRF24L01_Tx_SB(&data[0],32);  
 
  //If an ACK was not received, return an error
  if (check)
    return RF_ERROR_NO_ACK; 
     
  //Turn off the transmission indicator 
  PORTDbits.RD0 = 0;
  
  //Operation completed successfuly, return a 0  
  return 0x00;
}  

/*******************************************************************************
* Function:                                      
*                                                                             
* Variables:                        
* *data -> The data that is to be written to the display  
*                                                                             
* Description:                                                             
* This function will parse the data that is to be written to the BPT and send 
* it over two packets. The receiving BPT will then parse the data and display 
* the transmitted data as scrolling text. If the transmission is successful,
* the function will return a 0. Otherwise it will return an error code.                                   
*******************************************************************************/ 
UINT8 Tx_Scrolling_Text(UINT8 *data)
{ 
  UINT8 check;

  //Turn on the transmission indicator
  PORTDbits.RD0 = 1;  
   
  //Send the first half of the first USB packet
  check = nRF24L01_Tx_SB(&data[0],32);  
 
  //If an ACK was not received, return an error
  if (check)
    return RF_ERROR_NO_ACK; 
   
  //Send the second half of the first USB packet
  check = nRF24L01_Tx_SB(&data[32],32);  
 
  //If an ACK was not received, return an error
  if (check)
    return RF_ERROR_NO_ACK; 
      
  //Turn off the transmission indicator
  PORTDbits.RD0 = 0;
  
  //Operation completed successfuly, return a 0  
  return 0x00;
} 

#endif
