/*******************************************************************************
* Title: SPI_Setup.c                                                           *
* Version: 1.0                                                                 *
* Author: Jeff Nybo                                                            *
* Date: April 23, 2013                                                       *
*                                                                              *
* Description:                                                                 *
* This file contains the functions that allow the user to set up the SPI and   *
* send and receive data.                                                       *
*******************************************************************************/
            
#ifndef SPI_SETUP_C
#define SPI_SETUP_C

/*******************************************************************************
* Function: SPI_Init(void)                                                     *
*                                                                              *
* Variables:                                                                   *
* N/A                                                                          *
*                                                                              *
* Description:                                                                 *
* This function will initialize SPI and enable or disable it according to the  * 
* value of state. (Disable -> state = 0x00 : Enable -> state = 0x20)           *
*******************************************************************************/
void SPI_Init(uint16 setup)
{
  PIE1bits.SSPIE = 0;
  PIR1bits.SSPIF = 0;
  SSPSTAT = setup >> 8;       //SSPSTAT is equal to the high BYTE of setup
  SSPCON1 = 0x00FF & setup;   //SSPCON1 is equal to the low BYTE of setup 
  
  if (0x0020 & setup)
  {
   TRISCbits.TRISC7 = 0; 
   TRISBbits.TRISB0 = 1;
   TRISBbits.TRISB1 = 0;
  
   SSPCON1bits.SSPEN = 1;
  }  
}

/*******************************************************************************
* Function: SPI_Send(BYTE Tx)                                                  *
*                                                                              *
* Variables:                                                                   *
* Tx -> The value of the data that is to be transmitted                        *
*                                                                              *
* Description:                                                                 *
* This function will send the value of Tx while receiving and storing the data * 
* on SDI in Rx. The function will then return the value of Rx.                 *
*******************************************************************************/
BYTE SPI_Send(BYTE Tx) 
{
  //Reset SPI Flag
  PIR1bits.SSPIF = 0; 
  
  //Begin sending the data
  SSPBUF = Tx;
  
  //Wait for data to finish transmitting
  while (!SSPSTATbits.BF); 
    
  //Return received data
  return SSPBUF;
}

/*******************************************************************************
* Function: SPI_State(BYTE state)                                                                 * 
*                                                                             
* Variables:                                                                  
* state -> enables (1) or disables (0) the SPI module                         
*                                                                             
* Description:                                                                
* This function will allow the user to enable and disable the SPI module.    
*******************************************************************************/
void SPI_State(BYTE state)
{
  //If state is (0), disable SPI
  if (state == 0)
  {
    SSPCON1bits.SSPEN = 0; 
  }  
 
  //If state is (1), enable SPI
  else
  {
    TRISCbits.TRISC7 = 0; 
    TRISBbits.TRISB0 = 1;
    TRISBbits.TRISB1 = 0;
  
   SSPCON1bits.SSPEN = 1;
  }     
} 

#endif